@@ -9,7 +9,7 @@ use Types::Standard -types;
 use Capture::Tiny qw/capture/;
 use failures qw/module::comparison/;
 
-our $VERSION = '0.005';
+our $VERSION = '0.006';
 
 has perl_path => (is => 'ro', isa => Str, default => '');
 has modules => (is => 'ro', isa => ArrayRef);
@@ -66,20 +66,46 @@ sub check_local
     return $self->_run_mversion($cmd);
 }
 
+sub identify_resource
+{
+    my $self = shift;
+    my $identifier = shift;
+    if($identifier =~ m|docker://(.*)$|)
+    {
+        return ('docker', $1);
+    }
+    elsif($identifier =~ m|ssh://(.*)$|)
+    {
+        return ('ssh', $1);
+    }
+    elsif($identifier =~ m|/|)
+    {
+        # assume it's docker
+        return ('docker', $identifier);
+    }
+    else
+    {
+        # assume it's ssh
+        return ('ssh', $identifier);
+    }
+}
+
 sub check_correct_guess
 {
     my $self = shift;
     my $identifier = shift;
-    # perhaps add a special case for local?
-    if($identifier =~ m|/|)
+    my ($type, $server) = $self->identify_resource($identifier);
+    if($type eq 'ssh')
     {
-        # assume it's docker
-        return $self->check_container($identifier);
+        return $self->check_ssh_server($server);
+    }
+    elsif($type eq 'docker')
+    {
+        return $self->check_container($server);
     }
     else
     {
-        return $self->check_ssh_server($identifier);
-        # assume it's ssh
+        die 'I don\'t know what to do!';
     }
 }
 
@@ -121,6 +147,101 @@ sub _module_pair
     }
 }
 
+sub difference_report
+{
+    my $self = shift;
+    my $first = shift;
+    my $second = shift;
+
+    my %newer;
+    my %older;
+    my %newly_installed;
+    my %removed;
+    for my $module (keys %$first)
+    {
+        if(exists $second->{$module})
+        {
+            if($first->{$module} > $second->{$module})
+            {
+                $older{$module} = [$first->{$module}, $second->{$module}];
+            }
+            elsif($first->{$module} < $second->{$module})
+            {
+                $newer{$module} = [$first->{$module}, $second->{$module}];
+            }
+        }
+        else
+        {
+            $removed{$module} = $first->{$module};
+        }
+    }
+    for my $module (keys %$second)
+    {
+        unless(exists $first->{$module})
+        {
+            $newly_installed{$module} = $second->{$module};
+        }
+    }
+    return {
+        updated => \%newer,
+        downgraded => \%older,
+        removed => \%removed,
+        installed => \%newly_installed,
+    };
+}
+
+sub human_readable_report
+{
+	my $self = shift;
+    my $report = shift;
+    my @lines;
+    if(%{$report->{downgraded}})
+    {
+        push @lines, 'DOWNGRADED Modules', '';
+        for my $key (sort keys %{$report->{downgraded}}) 
+        {
+            my ($v1, $v2) = @{$report->{downgraded}->{$key}};
+            push @lines, sprintf("%-40s\t%s -> %s", $key, $v1, $v2);
+        }
+        push @lines, '';
+    }
+    if(%{$report->{removed}})
+    {
+        push @lines, 'REMOVED Modules', '';
+        for my $key (sort keys %{$report->{removed}}) 
+        {
+            my $v1 = $report->{removed}->{$key};
+            push @lines, sprintf("%-40s\t%s", $key, $v1);
+        }
+        push @lines, '';
+    }
+    if(%{$report->{installed}})
+    {
+        push @lines, 'Installed Modules', '';
+        for my $key (sort keys %{$report->{installed}}) 
+        {
+            my $v1 = $report->{installed}->{$key};
+            push @lines, sprintf("%-40s\t%s", $key, $v1);
+        }
+        push @lines, '';
+    }
+    if(%{$report->{updated}})
+    {
+        push @lines, 'Udated Modules', '';
+        for my $key (sort keys %{$report->{updated}}) 
+        {
+            my ($v1, $v2) = @{$report->{updated}->{$key}};
+            push @lines, sprintf("%-40s\t%s -> %s", $key, $v1, $v2);
+        }
+        push @lines, '';
+    }
+    unless(@lines)
+    {
+        push @lines, 'No differences', '';
+    }
+	return join "\n", @lines;
+}
+
 1;
 
 __END__
@@ -135,7 +256,7 @@ Server::Module::Comparison - check perl module versions installed on servers.
 
 =head1 VERSION
 
-version 0.005
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -193,6 +314,9 @@ Checks modules locally.
 This takes a string and guesses whether to assume it's a container identifier
 or an ssh server.
 
+Also supports a C<docker://> or C<ssh://> type uri scheme to help identify
+the server being identified.
+
 =head1 ATTRIBUTES
 
 =head2 perl_path
